package com.cf.cs.authentication.interceptor;

import com.cf.cs.base.misc.ApiContextUtils;
import com.cf.cs.base.misc.HelperUtils;
import com.cf.cs.protocol.misc.ProtocolJsonUtils;
import jakarta.servlet.ReadListener;
import jakarta.servlet.ServletInputStream;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletRequestWrapper;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

/**
 * The type Request body cache wrapper.
 *
 * @author xajhuang @163.com
 */
@Getter
@Slf4j
public class RequestBodyCacheWrapper extends HttpServletRequestWrapper {
    private final Long                reqTimeStamp = System.currentTimeMillis();
    private final Map<String, String> headers      = new HashMap<>();
    private       String              body;

    /**
     * Instantiates a new Request body cache wrapper.
     *
     * @param request the request
     * @throws IOException the io exception
     */
    public RequestBodyCacheWrapper(HttpServletRequest request) throws IOException {
        super(request);

        StringBuilder  stringBuilder  = new StringBuilder();
        BufferedReader bufferedReader = null;
        try (InputStream inputStream = request.getInputStream()) {
            if (inputStream != null) {
                bufferedReader = new BufferedReader(new InputStreamReader(inputStream, StandardCharsets.UTF_8));
                char[] charBuffer = new char[128];
                int    bytesRead;
                while ((bytesRead = bufferedReader.read(charBuffer)) > 0) {
                    stringBuilder.append(charBuffer, 0, bytesRead);
                }
            }
        } catch (Exception e) {
            log.error("RequestWrapper read error :{}", e.getMessage());
        } finally {

            if (bufferedReader != null) {
                bufferedReader.close();
            }
        }

        try {
            Object obj = ProtocolJsonUtils.jsonGetObject(stringBuilder.toString(), Object.class);
            body = HelperUtils.getJson(obj);

            Enumeration<String> enumeration = request.getHeaderNames();
            while (enumeration.hasMoreElements()) {
                String name  = enumeration.nextElement();
                String value = request.getHeader(name);
                headers.put(name, value);
            }
        } catch (Exception e) {
            body = "";
        } finally {
            ApiContextUtils.setRequestInfo(body, reqTimeStamp);
        }
    }

    @Override
    public ServletInputStream getInputStream() {
        final ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(body.getBytes());
        return new ServletInputStream() {
            @Override
            public boolean isFinished() {
                return false;
            }

            @Override
            public boolean isReady() {
                return false;
            }

            @Override
            public void setReadListener(ReadListener readListener) {
                // nothing to process
            }

            @Override
            public int read() {
                return byteArrayInputStream.read();
            }
        };
    }

    @Override
    public BufferedReader getReader() {
        return new BufferedReader(new InputStreamReader(this.getInputStream(), StandardCharsets.UTF_8));
    }
}
