package com.cf.cs.authentication.security.impl;

import com.cf.cs.authentication.misc.CasbinAuthorizeUtils;
import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.misc.ApiContextUtils;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.boot.autoconfigure.web.ServerProperties;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.authorization.AuthorizationDecision;
import org.springframework.security.authorization.AuthorizationManager;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.access.intercept.RequestAuthorizationContext;
import org.springframework.stereotype.Component;

import java.util.Optional;
import java.util.function.Supplier;

/**
 * The type Custom authorization manager.
 *
 * @author xajhuang @163.com
 */
@Component
public class CustomAuthorizationManager implements AuthorizationManager<RequestAuthorizationContext> {

    @Resource
    private CasbinAuthorizeUtils casbinAuthorizeUtils;

    @Resource
    private ServerProperties serverProperties;

    @Override
    public AuthorizationDecision check(Supplier<Authentication> authentications, RequestAuthorizationContext context) {
        // 从Supplier中获取Authentication对象
        Authentication auth = authentications.get();

        if (auth == null || !auth.isAuthenticated()) {
            throw new AccessDeniedException(ErrorCode.ERR_LOGOUT.getDescription());
        }

        // 获取用户名
        String username = ApiContextUtils.get().getUsername();

        // 从RequestAuthorizationContext获取HttpServletRequest
        HttpServletRequest request = context.getRequest();

        // 获取HTTP方法
        String httpMethod = request.getMethod();

        String prefixUrl = Optional.ofNullable(serverProperties.getServlet().getContextPath()).orElse("");
        String url       = request.getRequestURI().replaceFirst("^" + prefixUrl, "");

        return new AuthorizationDecision(casbinAuthorizeUtils.casbinVerifyAuthorize(username, url, httpMethod));
    }
}
