package com.cf.cs.authentication.security.impl;

import com.cf.cs.authentication.exception.CommonAuthException;
import com.cf.cs.authentication.pojo.po.AuthAccountUser;
import com.cf.cs.authentication.service.AuthUsersService;
import com.cf.cs.base.common.ErrorCode;
import jakarta.annotation.Resource;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.AuthorityUtils;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * The type User detail service.
 *
 * @author xajhuang @163.com
 */
@Service
public class UserDetailServiceImpl implements UserDetailsService {
    @Resource
    private AuthUsersService authUsersService;

    @Override
    public UserDetails loadUserByUsername(String username) {
        AuthAccountUser au = authUsersService.getAuthAccountByUserName(username);

        if (au == null) {
            throw new CommonAuthException(ErrorCode.ERR_USERNOTFOUND);
        }

        if (!au.isAccountNonLocked()) {
            throw new CommonAuthException(ErrorCode.ERR_USERLOCK);
        }

        if (!au.isEnabled()) {
            throw new CommonAuthException(ErrorCode.ERR_ACCOUNT);
        }

        au.setAuthorities(getUserAuthority(au.getUid()));
        return au;
    }

    /**
     * Gets user authority.
     *
     * @param userId the user id
     * @return the user authority
     */
    public List<GrantedAuthority> getUserAuthority(String userId) {
        return AuthorityUtils.commaSeparatedStringToAuthorityList(userId);
    }
}
