package com.cf.cs.base.config;

import com.cf.cs.base.common.CachePool;
import com.github.benmanes.caffeine.cache.Caffeine;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.cache.caffeine.CaffeineCache;
import org.springframework.cache.support.SimpleCacheManager;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * The type Cache configure.
 *
 * @author xajhuang @163.com
 */
@Configuration
@EnableCaching
public class CacheConfigure {
    /**
     * Cache manager cache manager.
     *
     * @return the cache manager
     */
    @Bean("caffeineCacheManager")
    public CacheManager cacheManager() {
        SimpleCacheManager  cacheManager = new SimpleCacheManager();
        List<CaffeineCache> list         = new ArrayList<>();
        //循环添加枚举类中自定义的缓存，可以自定义
        for (CachePool cacheEnum : CachePool.values()) {
            list.add(new CaffeineCache(cacheEnum.getCacheName(), Caffeine.newBuilder()
                                                                         .initialCapacity(cacheEnum.getDefaultPoolSize())
                                                                         .maximumSize(cacheEnum.getMaxPoolSize())
                                                                         .expireAfterAccess(cacheEnum.getDefaultTimeout(), TimeUnit.SECONDS)
                                                                         .build()));
        }
        cacheManager.setCaches(list);
        return cacheManager;
    }

}
