package com.cf.cs.base.config;

import jakarta.annotation.PostConstruct;
import jakarta.annotation.Resource;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.autoconfigure.web.ServerProperties;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

import java.net.InetAddress;

/**
 * The type Common configure.
 *
 * @author xajhuang @163.com
 */
@Data
@ConfigurationProperties(prefix = "common")
@Configuration
@Slf4j
@SuppressWarnings({"java:S3008", "java:S1444", "java:S1104"})
public class CommonConfigure {
    /**
     * The constant BASEURL.
     */
    public static String BASEURL;
    /**
     * The constant PROJECT_PREFIX_URL.
     */
    public static String PROJECT_PREFIX_URL;

    @Resource
    private ServerProperties serverProperties;
    private Integer tokenExpiredOfSeconds;
    private Integer allowPasswdRetryTimes;
    private boolean showSqlCommand;

    private static void setGlobalVars(String prefixUrl, String baseUrl) {
        PROJECT_PREFIX_URL = prefixUrl;
        BASEURL            = baseUrl;
    }

    @PostConstruct
    private void initGlobalValue() {
        log.info("Current: tokenExpiredOfSeconds = {}, allowPasswdRetryTimes = {}, showSqlCommand = {}",
                 tokenExpiredOfSeconds, allowPasswdRetryTimes, showSqlCommand);

        String addr = "localhost";
        try {
            addr = InetAddress.getLocalHost().getHostAddress();
        } catch (Exception e) {
            log.error("Unable get local ip address: {}", e.getMessage());
        } finally {
            setGlobalVars(serverProperties.getServlet().getContextPath(),
                          "http://" + addr + ":" + serverProperties.getPort() + serverProperties.getServlet().getContextPath());
            log.info("baseUrl: {}", BASEURL);
        }
    }
}
