package com.cf.cs.database.common;

import com.cf.cs.base.common.BaseEnum;
import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.JdbcType;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.List;

/**
 * The type Data base enum handler.
 *
 * @param <E> the type parameter
 * @author xajhuang @163.com
 */
public class DataBaseEnumHandler<E extends BaseEnum> extends BaseTypeHandler<E> {
    private final Class<E> enumType;
    private final List<E>  enums;

    /**
     * Instantiates a new Data base enum handler.
     *
     * @param type the type
     */
    public DataBaseEnumHandler(Class<E> type) {
        if (type == null) {
            throw new IllegalArgumentException("Type argument cannot be null");
        }

        this.enumType = type;
        this.enums    = Arrays.asList(type.getEnumConstants());
    }

    @Override
    public void setNonNullParameter(PreparedStatement preparedStatement, int i, E e, JdbcType jdbcType) throws SQLException {
        preparedStatement.setInt(i, e.getValue());
    }

    @Override
    public E getNullableResult(ResultSet resultSet, String s) throws SQLException {
        if (resultSet.getObject(s) == null) {
            return null;
        }

        int val = resultSet.getInt(s);
        return locateEnumStatus(val);
    }

    @Override
    public E getNullableResult(ResultSet resultSet, int index) throws SQLException {
        if (resultSet.getObject(index) == null) {
            return null;
        }

        int val = resultSet.getInt(index);
        return locateEnumStatus(val);
    }

    @Override
    public E getNullableResult(CallableStatement callableStatement, int index) throws SQLException {
        if (callableStatement.getObject(index) == null) {
            return null;
        }

        int val = callableStatement.getInt(index);
        return locateEnumStatus(val);
    }

    private E locateEnumStatus(int index) {
        for (E e : enums) {
            if (e.getValue() == index) {
                return e;
            }
        }
        throw new IllegalArgumentException(enumType.getName() + "  unknown enumerated type  index:" + index);
    }
}
