package com.cf.cs.database.service;


import com.cf.cs.database.pojo.entity.User;
import com.cf.cs.base.pojo.po.PageResults;
import com.cf.cs.database.pojo.po.ResPermInfo;
import com.cf.cs.database.pojo.po.UserInfo;
import com.mybatisflex.core.service.IService;

import java.util.List;

/**
 * 用户表(User)表服务接口
 *
 * @author makejava @163.com
 * @since 2023 -12-19 17:35:25
 */
public interface UserDataBaseService extends IService<User> {
    /**
     * Gets account user by name.
     *
     * @param username the username
     * @return the account user by name
     */
    User getAccountUserByName(String username);

    /**
     * Gets account user by uid.
     *
     * @param uid the uid
     * @return the account user by uid
     */
    User getAccountUserByUid(String uid);

    /**
     * Gets user info by id.
     *
     * @param userId the user id
     * @return the user info by id
     */
    UserInfo getUserInfoById(Long userId);

    /**
     * Gets user info by uid.
     *
     * @param uid the uid
     * @return the user info by uid
     */
    UserInfo getUserInfoByUid(String uid);

    /**
     * Gets user info by username.
     *
     * @param username the username
     * @return the user info by username
     */
    UserInfo getUserInfoByUsername(String username);

    /**
     * Gets current user info.
     *
     * @return the current user info
     */
    UserInfo getCurrentUserInfo();

    /**
     * Gets all user info.
     *
     * @param pageNumber the page number
     * @param pageSize   the page size
     * @param totalSize  the total size
     * @return the all user info
     */
    PageResults<UserInfo> getAllUserInfo(Long pageNumber, Long pageSize, Long totalSize);

    /**
     * Gets current user resource perm.
     *
     * @return the current user resource perm
     */
    List<ResPermInfo> getCurrentUserResourcePerm();

    /**
     * Gets user resource perm.
     *
     * @param userId the user id
     * @return the user resource perm
     */
    List<ResPermInfo> getUserResourcePerm(Long userId);
}

