package com.cf.cs.protocol.config;

import jakarta.annotation.PostConstruct;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

import java.util.Optional;

/**
 * The type Protocol configure.
 *
 * @author xajhuang @163.com
 */
@Data
@ConfigurationProperties(prefix = "protocol")
@Configuration
@Slf4j
@SuppressWarnings({"java:S3008", "java:S1444", "java:S1104"})
public class ProtocolConfigure {
    /**
     * The constant CHECK_TIMESTAMP.
     */
    public static boolean CHECK_TIMESTAMP;
    /**
     * The Timeout of seconds.
     */
    public static long    TIMEOUT_OF_SECONDS;
    /**
     * The Security protocol type.
     */
    public static int     SECURITY_PROTOCOL_TYPE;

    private Boolean checkTimestamp;
    private Integer timeoutOfSeconds;
    private Integer cryptoType;
    private String  cryptoKey;

    private static void setGlobalVars(boolean chkTimeStamp, long timeOfSecond, int secProType) {
        ProtocolConfigure.CHECK_TIMESTAMP        = chkTimeStamp;
        ProtocolConfigure.TIMEOUT_OF_SECONDS     = timeOfSecond;
        ProtocolConfigure.SECURITY_PROTOCOL_TYPE = secProType;
    }

    @PostConstruct
    private void initGlobalValue() {
        log.info("Current: checkTimestamp = {}, timeoutOfSeconds = {}", checkTimestamp, timeoutOfSeconds);

        setGlobalVars(Optional.ofNullable(checkTimestamp).orElse(false),
                      Optional.ofNullable(timeoutOfSeconds).orElse(30),
                      Optional.ofNullable(cryptoType).orElse(0));
    }

    /**
     * Gets timeout of millisecond.
     *
     * @return the timeout of millisecond
     */
    public Long getTimeoutOfMillisecond() {
        return (long) (timeoutOfSeconds * 1000);
    }
}
