package com.cf.cs.protocol.crypto;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpInputMessage;

import java.io.InputStream;
import java.nio.charset.StandardCharsets;

/**
 * The type Decrypt request protocol.
 *
 * @author xajhuang @163.com
 */
@Slf4j
public class DecryptRequestProtocol implements HttpInputMessage {

    private final HttpInputMessage inputMessage;

    private final String msgContent;

    /**
     * Instantiates a new Decrypt request protocol.
     *
     * @param inputMessage the input message
     * @param msgContent   the msg content
     */
    @Contract(pure = true)
    public DecryptRequestProtocol(HttpInputMessage inputMessage, String msgContent) {
        this.inputMessage = inputMessage;
        this.msgContent   = msgContent;
    }

    @Override
    @NotNull
    public InputStream getBody() {
        // 返回解密后的内容
        return IOUtils.toInputStream(msgContent, StandardCharsets.UTF_8);
    }

    @Override
    @NotNull
    public HttpHeaders getHeaders() {
        return inputMessage.getHeaders();
    }
}
