package com.cf.cs.protocol.crypto;


import com.cf.cs.crypto.annotation.DecryptionProtocol;
import com.cf.cs.protocol.service.ProtocolSecurityService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.core.MethodParameter;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.RequestBodyAdvice;

import java.io.IOException;
import java.lang.reflect.Type;

/**
 * The type Request protocol security.
 *
 * @author xajhuang @163.com
 */
@Slf4j
@RestControllerAdvice
public class RequestProtocolSecurity implements RequestBodyAdvice {

    @Resource
    private ProtocolSecurityService protocolSecurityService;

    @Override
    public boolean supports(@NotNull MethodParameter methodParameter,
                            @NotNull Type type,
                            @NotNull Class<? extends HttpMessageConverter<?>> aClass) {
        return methodParameter.getContainingClass().isAnnotationPresent(DecryptionProtocol.class)
                || methodParameter.hasMethodAnnotation(DecryptionProtocol.class);
    }

    @Override
    @NotNull
    public HttpInputMessage beforeBodyRead(@NotNull HttpInputMessage httpInputMessage,
                                           @NotNull MethodParameter methodParameter,
                                           @NotNull Type type,
                                           @NotNull Class<? extends HttpMessageConverter<?>> aClass) throws IOException {
        return protocolSecurityService.decryptProtocol(httpInputMessage);
    }

    @Override
    public Object handleEmptyBody(Object o,
                                  @NotNull HttpInputMessage httpInputMessage,
                                  @NotNull MethodParameter methodParameter,
                                  @NotNull Type type,
                                  @NotNull Class<? extends HttpMessageConverter<?>> aClass) {
        return o;
    }

    @Override
    @NotNull
    public Object afterBodyRead(@NotNull Object o,
                                @NotNull HttpInputMessage httpInputMessage,
                                @NotNull MethodParameter methodParameter,
                                @NotNull Type type,
                                @NotNull Class<? extends HttpMessageConverter<?>> aClass) {
        return o;
    }
}
