package com.cf.cs.protocol.pojo.po;

import com.cf.cs.base.common.ErrorCode;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;

/**
 * The type Opt log details resp.
 *
 * @author xajhuang @163.com
 */
@Schema(name = "系统操作日志详细信息")
@EqualsAndHashCode(callSuper = true)
@Data
@AllArgsConstructor
@NoArgsConstructor
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({"id", "requestHeaders", "request", "result", "status", "message"})
public class OptLogDetails extends BaseRespStatus {
    @Schema(description = "id")
    private Long id;

    @Schema(description = "HTTP 请求头所有参数")
    private String requestHeaders;

    @Schema(description = "操作请求参数")
    private String request;

    @Schema(description = "操作返回结果")
    private String result;

    /**
     * Instantiates a new Opt log details resp.
     *
     * @param id    the id
     * @param error the error
     */
    public OptLogDetails(Long id, ErrorCode error) {
        this.id = id;
        this.setStatus(error.getCode());
        this.setMessage(new String[] {error.getDescription()});
    }
}
