package com.cf.cs.protocol.pojo.vo;

import com.cf.cs.base.common.CommonEnumHandler;
import com.cf.cs.base.common.ConstValue;
import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.protocol.config.ProtocolConfigure;
import com.cf.cs.protocol.pojo.base.BaseProtocol;
import com.cf.cs.protocol.pojo.po.BaseRespStatus;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/**
 * The type Protocol resp.
 *
 * @param <T> the type parameter
 * @author xajhuang @163.com
 */
@Slf4j
@EqualsAndHashCode(callSuper = true)
@Data
@NoArgsConstructor
@AllArgsConstructor
@JsonPropertyOrder({"ver", "cryptoType", "timeStamp", "code", "msgContent"})
@JsonInclude(JsonInclude.Include.NON_NULL)
public class ProtocolResp<T> extends BaseProtocol<T> {
    private Integer code;

    private static <T> ProtocolResp<T> result(Integer httpCode, T respMsg) {
        ProtocolResp<T> resp = new ProtocolResp<>();

        resp.setVer(ConstValue.Protocol.VERSION);
        resp.setCode(httpCode);
        resp.setCryptoType(ProtocolConfigure.SECURITY_PROTOCOL_TYPE);
        resp.setTimeStamp(System.currentTimeMillis());
        resp.setMsgContent(respMsg);
        return resp;
    }

    /**
     * Result protocol resp.
     *
     * @param <T>     the type parameter
     * @param err     to err
     * @param respMsg the resp msg
     * @return the protocol resp
     */
    public static <T> ProtocolResp<T> result(ErrorCode err, T respMsg) {
        return result(err.getHttpCode(), respMsg);
    }

    /**
     * Result protocol resp.
     *
     * @param err to err
     * @return the protocol resp
     */
    public static ProtocolResp<BaseRespStatus> result(ErrorCode err) {
        BaseRespStatus rspMsg = new BaseRespStatus(err.getCode(), new String[] {err.getDescription()});
        return result(err.getHttpCode(), rspMsg);
    }

    /**
     * Result protocol resp.
     *
     * @param <T>     the type parameter
     * @param respMsg the resp msg
     * @return the protocol resp
     */
    public static <T extends BaseRespStatus> ProtocolResp<T> result(T respMsg) {
        return result(CommonEnumHandler.codeOf(ErrorCode.class, respMsg.getStatus()), respMsg);
    }

    /**
     * Result protocol resp.
     *
     * @param errCode  the err code
     * @param httpCode the http code
     * @param message  the message
     * @return the protocol resp
     */
    public static ProtocolResp<BaseRespStatus> result(ErrorCode errCode, Integer httpCode, String[] message) {
        BaseRespStatus rspMsg = new BaseRespStatus(errCode.getCode(), message);
        return result(httpCode, rspMsg);
    }
}
