package com.cf.cs.protocol.validation.valids.impl;


import com.cf.cs.base.common.ConstValue;
import com.cf.cs.protocol.config.ProtocolConfigure;
import com.cf.cs.protocol.validation.valids.ValidProtocolTimestamp;
import jakarta.validation.ConstraintValidator;
import jakarta.validation.ConstraintValidatorContext;

import java.text.SimpleDateFormat;


/**
 * The type Valid protocol timestamp.
 *
 * @author xajhuang @163.com
 */
public class ValidProtocolTimestampImpl implements ConstraintValidator<ValidProtocolTimestamp, Long> {

    @Override
    public void initialize(ValidProtocolTimestamp constraintAnnotation) {
        ConstraintValidator.super.initialize(constraintAnnotation);
    }

    @Override
    public boolean isValid(Long timeStamp, ConstraintValidatorContext constraintValidatorContext) {

        if (!ProtocolConfigure.CHECK_TIMESTAMP) {
            return true;
        }

        SimpleDateFormat format  = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
        String           date    = format.format(timeStamp);
        String           current = format.format(System.currentTimeMillis());


        if (timeStamp > System.currentTimeMillis()) {
            String errMsg = String.format("请求时间[%s] 大于当前服务器时间 [%s]", date, current);
            //禁用默认的message的值
            constraintValidatorContext.disableDefaultConstraintViolation();
            //重新添加错误提示语句
            constraintValidatorContext.buildConstraintViolationWithTemplate(errMsg).addConstraintViolation();
            return false;
        } else if (System.currentTimeMillis() - timeStamp > ProtocolConfigure.TIMEOUT_OF_SECONDS * ConstValue.MS_OF_SECONDS) {
            String errMsg = String.format("请求时间[%s] 延时超过当前服务器时间[%s] 允许范围 %d(S)", date, current,
                                          ProtocolConfigure.TIMEOUT_OF_SECONDS);
            //禁用默认的message的值
            constraintValidatorContext.disableDefaultConstraintViolation();
            //重新添加错误提示语句
            constraintValidatorContext.buildConstraintViolationWithTemplate(errMsg).addConstraintViolation();
            return false;
        }

        return true;
    }
}
