package com.cf.cs.restful.controller;

import com.cf.cs.base.annotation.OperationLogAnnotation;
import com.cf.cs.crypto.annotation.DecryptionProtocol;
import com.cf.cs.crypto.annotation.EncryptionProtocol;
import com.cf.cs.protocol.pojo.po.VersionInfo;
import com.cf.cs.protocol.pojo.vo.ProtocolResp;
import com.cf.cs.protocol.pojo.vo.VersionResp;
import com.cf.cs.restful.configure.ProjectGitVersionInfo;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * The type Common framework api.
 *
 * @author xajhuang @163.com
 */
@RestController
@Slf4j
@Tag(name = "通用接口")
@RequestMapping(value = "/api")
public class CommonFrameworkApi {
    @Resource
    private ProjectGitVersionInfo prgVer;

    /**
     * Gets version.
     *
     * @return the version
     */
    @EncryptionProtocol
    @DecryptionProtocol
    @PostMapping("/version")
    @OperationLogAnnotation(OperationModule = "通用模块", OperationType = "读取", OperationDesc = "获取当前系统版本信息")
    public ProtocolResp<VersionResp> getVersion() {
        return ProtocolResp.result(VersionResp.builder()
                                              .version(VersionInfo.builder()
                                                                  .commitId(prgVer.getCommitId())
                                                                  .commitDescribe(prgVer.getCommitDescribe())
                                                                  .commitTime(prgVer.getCommitTime())
                                                                  .tagName(prgVer.getTagName())
                                                                  .buildTime(prgVer.getBuildTime())
                                                                  .gitBranch(prgVer.getGitBranch())
                                                                  .build())
                                              .build());
    }

    /**
     * Gets version v 2.
     *
     * @return the version v 2
     */
    @GetMapping("/version")
    @EncryptionProtocol
    @OperationLogAnnotation(OperationModule = "通用模块", OperationType = "读取", OperationDesc = "获取当前系统版本信息")
    public ProtocolResp<VersionResp> getVersionV2() {
        return ProtocolResp.result(VersionResp.builder()
                                              .version(VersionInfo.builder()
                                                                  .commitId(prgVer.getCommitId())
                                                                  .commitDescribe(prgVer.getCommitDescribe())
                                                                  .commitTime(prgVer.getCommitTime())
                                                                  .tagName(prgVer.getTagName())
                                                                  .buildTime(prgVer.getBuildTime())
                                                                  .gitBranch(prgVer.getGitBranch())
                                                                  .build())
                                              .build());
    }
}
