package com.cf.cs.restful.controller;


import com.cf.cs.base.annotation.OperationLogAnnotation;
import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.base.misc.HelperUtils;
import com.cf.cs.crypto.annotation.EncryptionProtocol;
import com.cf.cs.base.pojo.po.PageResults;
import com.cf.cs.protocol.pojo.dto.OperationLogDetailsReq;
import com.cf.cs.protocol.pojo.dto.OperationLogReq;
import com.cf.cs.protocol.pojo.dto.ProtocolReq;
import com.cf.cs.protocol.pojo.po.BaseRespStatus;
import com.cf.cs.protocol.pojo.po.OptLogDetails;
import com.cf.cs.protocol.pojo.vo.BasePageResultResp;
import com.cf.cs.protocol.pojo.vo.OperationLogDetailsResp;
import com.cf.cs.protocol.pojo.vo.OptLogSummaryResp;
import com.cf.cs.protocol.pojo.vo.ProtocolResp;
import com.cf.cs.protocol.validation.group.ValidGroups;
import com.cf.cs.restful.service.OperationLogService;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * The type Operation log api.
 *
 * @author xajhuang @163.com
 */
@RestController
@Slf4j
@Tag(name = "操作日志接口")
@RequestMapping(value = "/api/operation")
@SuppressWarnings("java:S1452")
public class OperationLogApi {
    @Resource
    private OperationLogService operationLogService;

    /**
     * Gets summary.
     *
     * @param mr the mr
     * @return the summary
     */
    @PostMapping("/summary")
    @EncryptionProtocol
    @OperationLogAnnotation(OperationModule = "操作日志模块", OperationType = "读取", OperationDesc = "获取操作日志简要信息")
    public ProtocolResp<? extends BaseRespStatus> getSummary(@RequestBody ProtocolReq<OperationLogReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class,
                                                     ValidGroups.BasePagedReqValid.class,
                                                     ValidGroups.OperationLogReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            OperationLogReq req = mr.getMsgContent();

            PageResults<OptLogSummaryResp> ret = operationLogService.getOperationLogSummary(req.getPageNumber(), req.getPageSize(),
                                                                                            req.getTotalSize(), req.getUserName());

            return ProtocolResp.result(BasePageResultResp.<OptLogSummaryResp>builder()
                                                         .items(ret)
                                                         .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION,
                                       ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }

    /**
     * Gets details.
     *
     * @param mr the mr
     * @return the details
     */
    @PostMapping("/details")
    @EncryptionProtocol
    @OperationLogAnnotation(OperationModule = "操作日志模块", OperationType = "读取", OperationDesc = "获取操作日志详细信息")
    public ProtocolResp<? extends BaseRespStatus> getDetails(@RequestBody ProtocolReq<OperationLogDetailsReq> mr) {
        List<String> validate = HelperUtils.validate(mr, ValidGroups.ProtocolCommonValid.class,
                                                     ValidGroups.OperationLogReqValid.class);
        // 如果校验通过，validate为空；否则，validate包含未校验通过项
        if (validate.isEmpty()) {
            OperationLogDetailsReq req = mr.getMsgContent();

            List<OptLogDetails> ret = operationLogService.getOperationLogDetails(req.getOperationIds());

            return ProtocolResp.result(OperationLogDetailsResp.builder()
                                                              .operationLog(ret)
                                                              .build());
        } else {
            return ProtocolResp.result(ErrorCode.ERR_PARAMEXCEPTION,
                                       ErrorCode.ERR_PARAMEXCEPTION.getHttpCode(),
                                       validate.toArray(new String[0]));
        }
    }
}
