package com.cf.cs.restful.pojo.mapper;

import com.cf.cs.database.pojo.entity.OperationLog;
import com.cf.cs.protocol.pojo.po.HwFirmware;
import com.cf.cs.protocol.pojo.po.HwInfo;
import com.cf.cs.protocol.pojo.po.HwMotherBoard;
import com.cf.cs.protocol.pojo.po.OptLogDetails;
import com.cf.cs.protocol.pojo.po.ProcessorInfo;
import com.cf.cs.protocol.pojo.vo.OptLogSummaryResp;
import org.mapstruct.IterableMapping;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Mappings;
import org.mapstruct.Named;
import org.mapstruct.factory.Mappers;
import oshi.hardware.Baseboard;
import oshi.hardware.CentralProcessor;
import oshi.hardware.ComputerSystem;
import oshi.hardware.Firmware;

import java.util.List;

/**
 * The interface Object convert.
 *
 * @author xajhuang @163.com
 */
@Mapper
@SuppressWarnings("java:S1710")
public interface IObjectConvert {
    /**
     * The constant INSTANCE.
     */
    IObjectConvert INSTANCE = Mappers.getMapper(IObjectConvert.class);

    /**
     * To opt log summary resp opt log summary resp.
     *
     * @param optLog the opt log
     * @return the opt log summary resp
     */
    List<OptLogSummaryResp> toOptLogSummaryResp(List<OperationLog> optLog);

    /**
     * To opt log details resp opt log details resp.
     *
     * @param optLog the opt log
     * @return the opt log details resp
     */
    @IterableMapping(qualifiedByName="toOptLogDetails")
    List<OptLogDetails> toOptLogDetailsArray(List<OperationLog> optLog);

    /**
     * To opt log details opt log details.
     *
     * @param optLog the opt log
     * @return the opt log details
     */
    @Named("toOptLogDetails")
    @Mappings(value = {
            @Mapping(target = "status", ignore = true),
            @Mapping(target = "message", ignore = true)
    })
    OptLogDetails toOptLogDetails(OperationLog optLog);

    /**
     * To hw firmware.
     *
     * @param f the f
     * @return the hw firmware
     */
    @Mappings({
            @Mapping(expression = "java(f.getName())", target = "name"),
            @Mapping(expression = "java(f.getVersion())", target = "version"),
            @Mapping(expression = "java(f.getDescription())", target = "description"),
            @Mapping(expression = "java(f.getManufacturer())", target = "manufacturer"),
            @Mapping(expression = "java(f.getReleaseDate())", target = "releaseDate")
    })
    HwFirmware toHwFirmware(Firmware f);

    /**
     * To hw mother board hw mother board.
     *
     * @param b the b
     * @return the hw mother board
     */
    @Mappings({
            @Mapping(expression = "java(b.getVersion().trim())", target = "version"),
            @Mapping(expression = "java(b.getManufacturer())", target = "manufacturer"),
            @Mapping(expression = "java(b.getModel())", target = "model"),
            @Mapping(expression = "java(b.getSerialNumber())", target = "serialNumber")
    })
    HwMotherBoard toHwMotherBoard(Baseboard b);

    /**
     * To hw info.
     *
     * @param c the c
     * @return the hw info
     */
    @Mappings({
            @Mapping(expression = "java(c.getModel())", target = "model"),
            @Mapping(expression = "java(c.getManufacturer())", target = "manufacturer"),
            @Mapping(expression = "java(c.getHardwareUUID())", target = "uuid"),
            @Mapping(expression = "java(c.getSerialNumber())", target = "serialNumber"),
            @Mapping(expression = "java(toHwFirmware(c.getFirmware()))", target = "firmware"),
            @Mapping(expression = "java(toHwMotherBoard(c.getBaseboard()))", target = "mb")
    })
    HwInfo toHwInfo(ComputerSystem c);

    /**
     * To processor info processor info.
     *
     * @param p the p
     * @param i the
     * @return the processor info
     */
    @Mappings({
            @Mapping(expression = "java(p.getProcessorIdentifier().getVendor())", target = "cpuVendor"),
            @Mapping(expression = "java(p.getProcessorIdentifier().getName())", target = "cpuName"),
            @Mapping(expression = "java(p.getProcessorIdentifier().getProcessorID())", target = "processorId"),
            @Mapping(expression = "java(p.getProcessorIdentifier().getIdentifier())", target = "cpuIdentifier"),
            @Mapping(expression = "java(p.getProcessorIdentifier().getMicroarchitecture())", target = "microArchitecture"),
            @Mapping(expression = "java(p.getProcessorIdentifier().isCpu64bit())", target = "cpu64bit"),
            @Mapping(expression = "java(p.getMaxFreq())", target = "cpuVendorFreq"),
            @Mapping(expression = "java(p.getPhysicalPackageCount())", target = "physicalCpus"),
            @Mapping(expression = "java(p.getPhysicalProcessorCount())", target = "physicalCores"),
            @Mapping(expression = "java(p.getLogicalProcessorCount())", target = "logicalCpu")
    })
    ProcessorInfo toProcessorInfo(CentralProcessor p, CentralProcessor.ProcessorIdentifier i);
}
