package com.cf.cs.restful.service.impl;

import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.database.pojo.entity.OperationLog;
import com.cf.cs.base.pojo.po.PageResults;
import com.cf.cs.database.service.OperationLogDataBaseService;
import com.cf.cs.protocol.pojo.po.OptLogDetails;
import com.cf.cs.protocol.pojo.vo.OptLogSummaryResp;
import com.cf.cs.restful.pojo.mapper.IObjectConvert;
import com.cf.cs.restful.service.OperationLogService;
import com.mybatisflex.core.paginate.Page;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * The type Operation log service.
 *
 * @author xajhuang @163.com
 */
@Service
public class OperationLogServiceImpl implements OperationLogService {
    @Resource
    private OperationLogDataBaseService operationLogDataBaseService;

    @Override
    public PageResults<OptLogSummaryResp> getOperationLogSummary(Long pageNumber, Long pageSize, Long totalSize, List<String> userName) {
        if (totalSize == 0) {
            totalSize = -1L;
        }

        Page<OperationLog> optLogs = operationLogDataBaseService.getSystemOperationSummary(pageNumber, pageSize, totalSize, userName);

        return PageResults.<OptLogSummaryResp>builder()
                          .items(IObjectConvert.INSTANCE.toOptLogSummaryResp(optLogs.getRecords()))
                          .pageNumber(optLogs.getPageNumber())
                          .pageSize(optLogs.getPageSize())
                          .totalPage(optLogs.getTotalPage())
                          .totalRow(optLogs.getTotalRow())
                          .build();
    }

    @Override
    public List<OptLogDetails> getOperationLogDetails(List<Long> optLogIds) {
        List<OperationLog>  optLogs = operationLogDataBaseService.getSystemOperationDetails(optLogIds);
        List<OptLogDetails> ret     = IObjectConvert.INSTANCE.toOptLogDetailsArray(optLogs);
        List<OptLogDetails> notFound = optLogIds.stream()
                                                .filter(k -> !ret.stream()
                                                                       .map(OptLogDetails::getId)
                                                                       .toList().contains(k))
                                                .map(m -> new OptLogDetails(m, ErrorCode.ERR_NOSUCHITEM))
                                                .toList();
        ret.forEach(k -> {
            k.setMessage(new String[] {ErrorCode.ERR_OK.getDescription()});
            k.setStatus(ErrorCode.ERR_OK.getCode());
        });

        ret.addAll(notFound);
        return ret;
    }
}
