package com.cf.cs.restful.service.impl;


import com.cf.cs.base.common.ErrorCode;
import com.cf.cs.database.mapper.ResourceDataMapper;
import com.cf.cs.database.mapper.ResourceTypeMapper;
import com.cf.cs.database.mapper.RoleMapper;
import com.cf.cs.database.mapper.RoleResourceMapper;
import com.cf.cs.database.pojo.entity.ResourceData;
import com.cf.cs.database.pojo.entity.Role;
import com.cf.cs.database.pojo.entity.RoleResource;
import com.cf.cs.protocol.pojo.po.ResOperationStatus;
import com.cf.cs.protocol.pojo.po.ResourceInfo;
import com.cf.cs.restful.service.UserPermissionService;
import com.mybatisflex.core.query.QueryWrapper;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.cf.cs.database.pojo.entity.table.ResourceDataTableDef.RESOURCE_DATA;
import static com.cf.cs.database.pojo.entity.table.RoleResourceTableDef.ROLE_RESOURCE;

/**
 * The type User permission service.
 *
 * @author xajhuang @163.com
 */
@Service
public class UserPermissionServiceImpl implements UserPermissionService {
    private final Map<Integer, String> resourceType = new HashMap<>();

    @Resource
    private ResourceDataMapper resDataMapper;

    @Resource
    private RoleResourceMapper roleResourceMapper;

    @Resource
    private RoleMapper roleMapper;


    @Resource
    private ResourceTypeMapper resourceTypeMapper;

    @Override
    public List<Role> getAllUserGroup() {
        return roleMapper.selectAll();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<ResOperationStatus> registerResource(List<ResourceInfo> resources) {
        List<ResOperationStatus> result = new ArrayList<>();

        resources.forEach(k -> {
            if (!resourceType.containsKey(k.getResTypeId())) {
                result.add(new ResOperationStatus(k.getPath(),
                                                  String.join("|", k.getHttpMethod()),
                                                  k.getResTypeId(),
                                                  ErrorCode.ERR_NOSUCHITEM));
                return;
            }
            k.getHttpMethod().forEach(v -> {
                QueryWrapper wrapper = QueryWrapper.create()
                                                   .from(RESOURCE_DATA)
                                                   .select()
                                                   .where(RESOURCE_DATA.PATH.eq(k.getPath()))
                                                   .and(RESOURCE_DATA.HTTP_METHOD.like("%" + v + "%"));

                if (resDataMapper.selectOneByQuery(wrapper) != null) {
                    result.add(new ResOperationStatus(k.getPath(), v, k.getResTypeId(), ErrorCode.ERR_ITEMEXISTS));
                } else {
                    ResourceData res = new ResourceData();
                    res.setHttpMethod(v);
                    res.setTypeId(k.getResTypeId());
                    res.setName(k.getName());
                    res.setPath(k.getPath());
                    if (resDataMapper.insert(res) == 1) {
                        result.add(new ResOperationStatus(res.getUid(), k.getPath(), v, k.getResTypeId(), ErrorCode.ERR_OK));
                    } else {
                        result.add(new ResOperationStatus(k.getPath(), v, k.getResTypeId(), ErrorCode.ERR_DATABASE));
                    }
                }
            });
        });

        return result;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<ResOperationStatus> removeResourceByUid(List<String> ids) {
        List<ResOperationStatus> result = new ArrayList<>();

        ids.forEach(k -> {
            ResourceData res = resDataMapper.selectOneByCondition(RESOURCE_DATA.UID.eq(k));

            if (res == null) {
                result.add(new ResOperationStatus(k, ErrorCode.ERR_NOSUCHITEM));
                return;
            }

            QueryWrapper wrapper = QueryWrapper.create()
                                               .from(ROLE_RESOURCE)
                                               .select()
                                               .where(ROLE_RESOURCE.RESOURCE_ID.eq(res.getId()));

            RoleResource role = roleResourceMapper.selectOneByQuery(wrapper);

            if (role != null) {
                result.add(new ResOperationStatus(k, ErrorCode.ERR_RESOURCE_USED));
                return;
            }

            if (resDataMapper.deleteByCondition(RESOURCE_DATA.UID.eq(k)) == 1) {
                result.add(new ResOperationStatus(k, ErrorCode.ERR_OK));
            } else {
                result.add(new ResOperationStatus(k, ErrorCode.ERR_DATABASE));
            }
        });

        return result;
    }


    /**
     * Init.
     */
    @PostConstruct
    public void init() {
        // 在组件初始化后执行的逻辑
        resourceTypeMapper.selectAll().forEach(k -> {
            if (!resourceType.containsKey(k.getTypeId())) {
                resourceType.put(k.getTypeId(), k.getName());
            }
        });
        // 可以在这里进行一些初始化操作，如初始化参数、加载数据等
    }
}
